import React, { useEffect, useMemo, useRef, useState } from 'react'
import styles from './index.less'
import { Table, Tooltip, message, Button, Pagination } from 'antd'
import SearchGroup from './components/SearchGroup'
import TablePack from './components/TablePack'
import { GetAccountConfigInfo, GetAccountPageList, getStationListByUserID, GetTableJson, SaveTableDataInfo } from '../../apis/process'
import { isJson, isObject } from '../../utils/index'
import IconPack from '../widgets/IconPack'
import FormRender from '../FormRender'

const getFileInfo = (formJson) => {
  let obj = {}
  let json = isJson(formJson) ? JSON.parse(formJson) : {}
  let parent = json?.properties
  if (isObject(parent)) {
    for (let v in parent) {
      let child = parent[v]?.properties
      if (isObject(child)) {
        for (let s in child) {
          obj[s] = child[s]
        }
      }
    }
  }
  return obj
}

const TableRender = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { accountName } = props
  const [detailShow, setDetailShow] = useState(false)
  const [params, setParams] = useState({ userID: userID, accountName, direction: 'desc', timeField: '录入时间', total: 0, pageIndex: 1, pageSize: 100 })
  const [loading, setLoading] = useState(false)
  const [dataSource, setDataSource] = useState([])
  const [config, setConfig] = useState({ webShowFieldGroup: '', addFieldGroup: '', formJson: {} })
  const [schemaValues, setSchemaValues] = useState({ formJson: {}, values: [] })
  const [submitLoading, setSubmitLoading] = useState(false)
  const formRenderRef = useRef()

  const fileColumns = useMemo(() => {
    const { webShowFieldGroup, formJson } = config
    let showField = webShowFieldGroup ? webShowFieldGroup.split(',').filter(v => v) : []
    let json = getFileInfo(formJson)
    let array = []
    showField.forEach(v => {
      array.push({
        title: json[v]?.title || v,
        dataIndex: v,
        width: 120,
        ellipsis: true,
        render: (value, r) => {
          let props = { ...json[v], value }
          return (
            <TablePack {...props} />
          )
        }
      })
    })
    return array
  }, [config])

  const columns = useMemo(() => {
    return [
      {
        title: '序号',
        dataIndex: 'r',
        key: 'r',
        width: 50,
        align: 'center',
        fixed: 'left',
      },
      ...fileColumns,
      {
        title: '操作',
        width: 120,
        align: 'center',
        fixed: 'right',
        render: (r) => {
          let btns = [
            {
              title: '详情',
              icon: <IconPack.XiangQing />,
            },
            {
              title: '编辑',
              icon: <IconPack.BianJi />,
            },
            {
              title: '删除',
              icon: <IconPack.ShanChu />,
            }
          ]
          return (
            <div style={{ display: 'flex', justifyContent: 'center' }}>
              {
                btns.map(v => {
                  return (
                    <Button
                      type='link'
                      title={v.title}
                      icon={v.icon}
                      style={{ padding: '0 7px' }}
                      onClick={() => btnsClick(v.title, r)}
                    >
                    </Button>
                  )
                })
              }
            </div>
          )
        }
      }
    ]
  }, [fileColumns])

  const btnsClick = (type, row) => {
    const { addFieldGroup, editFieldGroup, formJson } = config
    const addField = addFieldGroup ? addFieldGroup.split(',').filter(v => v) : []
    if (type === '添加') {
      addClick(addField, formJson)
    }
    // if (['添加', '详情', '编辑'].includes(type)) {
    //   setDetailShow(true)
    // }
  }

  const addClick = (addField, formJson) => {
    let json = JSON.parse(JSON.stringify(formJson))
    let parent = json?.properties
    if (isObject(parent)) {
      for (let v in parent) {
        let hidden = true
        let child = parent[v]?.properties
        if (isObject(child)) {
          for (let s in child) {
            if (addField.includes(s)) {
              child[s].hidden = false
              hidden = false
            } else {
              child[s].hidden = true
            }
          }
        }
        parent[v].hidden = hidden
      }
    }
    setSchemaValues({ formJson: json, values: [] })
    setDetailShow(true)
  }

  const submit = async () => {
    const { formValue, relationForm, errors } = await formRenderRef.current.getValues()
    if (errors.length) {
      return message.error('请完善表单内容')
    }
    setSubmitLoading(true)
    const { code, data } = await SaveTableDataInfo({ accountTable: accountName, id: null, values: formValue })
    if (code === 0) {
      message.success('保存成功!')
      setSubmitLoading(false)
      setDetailShow(false)
    }
  }

  const search = (values) => {
    setParams({ ...params, ...values })
    getDataSource({ ...params, ...values })
  }

  const getConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName)
    const res = await GetTableJson(accountName)
    if (code === 0 && res.code === 0) {
      setConfig({ ...data, formJson: isJson(res.data) ? JSON.parse(res.data) : {} })
    }
  }

  const getDataSource = async (param = {}) => {
    setLoading(true)
    const { code, data, msg } = await GetAccountPageList({ ...params, ...param })
    if (code === 0) {
      setDataSource(JSON.parse(data.jsonData))
      setParams({ ...params, total: data.totalCount, pageIndex: data.pageIndex, pageSize: data.pageSize })
    } else {
      setDataSource([])
      message.error(msg)
    }
    setLoading(false)
  }

  const pageChange = (pageIndex, pageSize) => {
    getDataSource({ pageIndex, pageSize })
  }

  const getData = () => {
    getConfig()
    getDataSource()
  }

  useEffect(() => {
    getData()
  }, [])

  return (
    <div className={styles.account}>
      <div className={styles.tableRender} style={{ display: !detailShow ? 'block' : 'none' }}>
        <div className={styles.content}>
          <div className={styles.top}>
            <SearchGroup
              onChange={search}
              btnsClick={btnsClick}
            />
          </div>
          <div className={styles.bottom}>
            <Table
              size='small'
              rowKey='ID'
              bordered
              loading={loading}
              columns={columns}
              dataSource={dataSource}
              pagination={false}
              scroll={{ y: '100%' }}
            />
          </div>
        </div>
        <div className={styles.footer}>
          <div className={styles.total}>共计{params.total}条数据</div>
          <div className={styles.pagination}>
            <Pagination
              showQuickJumper
              pageSize={params.pageSize}
              current={params.pageIndex}
              total={params.total}
              onChange={pageChange}
            />
          </div>
        </div>
      </div>
      <div className={styles.tableDetail} style={{ display: detailShow ? 'block' : 'none' }}>
        <div className={styles.formBox}>
          <FormRender ref={formRenderRef} schemaValues={schemaValues} />
        </div>
        <div className={styles.formBtns}>
          <Button
            onClick={() => setDetailShow(false)}
          >
            返回
          </Button>
          <Button
            type='primary'
            loading={submitLoading}
            onClick={submit}
          >
            提交
          </Button>
        </div>
      </div>
    </div>

  )

}

export default TableRender